﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using System;

  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Integer;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.String;

  public class Grades : BenchmarkSuiteDataDescriptor {
    private const string name = "Grades - Hard";
    private const string fileName = "Grades.csv";
    private const string description = "Given 5 integers, the first four represent the lower numeric thresholds for achieving an A, B, C, and D, and will be distinct and in descending order. The fifth represents the student’s numeric grade. The program must print Student has a X grade., where X is A, B, C, D, or F depending on the thresholds and the numeric grade.";

    protected override string FileName { get { return fileName; } }
    public override string Name { get { return name; } }
    public override string Description { get { return description; } }
    protected override int InputArgumentCount { get { return 5; } }
    protected override int OutputArgumentCount { get { return 1; } }

    public override ProblemData CreateProblemData() {
      return new ProblemData(ProblemType.Grades) {
        Name = Name,
        Description = Description,
        ProgramExecutionBudget = 60000000,
        Examples = CloneExamples(),
        BestResult = 0,
        WorstResult = 5000,
        InputArgumentTypes = new[] { ExampleArgumentType.Integer, ExampleArgumentType.Integer, ExampleArgumentType.Integer, ExampleArgumentType.Integer, ExampleArgumentType.Integer },
        OutputArgumentTypes = new[] { ExampleArgumentType.Print },
        TrainingCount = 200,
        TestCount = 2000,
        EnabledDataTypes = DataTypes.Exec | DataTypes.Integer | DataTypes.Boolean | DataTypes.String | DataTypes.Print,
        MaxSize = 400,
        EvalLimit = 800,
        ErcOptions = {
          ErcProbability = 0.05,
          IntegerErcOptions = new IntegerErcOptions(
            new IntegerRangeErc(0, 100)),
          StringErcOptions = new StringErcOptions(
            new StringConstantErc("Student has a ", " grade", "A", "B", "C", "D", "F"))
        }
      };
    }

    protected override Example ParseExample(string[] input, string[] output) {
      char grade;
      if (!GetGrade(output[0], out grade))
        throw new InvalidOperationException("Unable to parse grade.");

      return new Example {
        InputArgs = input,
        OutputArgs = output,
        InputInteger = ExampleArgumentConverter.ConvertIntegers(input),
        OutputPrint = output[0],

        // help
        OutputChar = new[] { grade }
      };
    }

    public static bool GetGrade(string str, out char grade) {
      const string start = "Student has a ";

      if (str.StartsWith(start) && str.Length > start.Length) {
        grade = str[start.Length];
        return true;
      }

      grade = default(char);
      return false;
    }
  }
}
